/* HEPv3 functions */

#include "rtpPlugin.c"

int send_json_hepv3(FlowHashBucket *bkt, time_t when,
		    char *data, unsigned int data_len) {
  struct hep_generic *hg=NULL;
  char buffer[2048];
  unsigned int buflen=0, iplen=0, tlen=0;
  hep_chunk_ip4_t src_ip4, dst_ip4;
#ifdef USE_IPV6
  hep_chunk_ip6_t src_ip6, dst_ip6;
#endif
  hep_chunk_t payload_chunk;
  hep_chunk_t correlation_chunk;
  hep_chunk_t authkey_chunk;
  u_int correlation_id_len, max_len;
  char *correlation_id = NULL;
  char buf[64];
  unsigned int sendzip = 0;

  max_len = sizeof(buffer)-148;

  if(readOnlyGlobals.hep.pwd != NULL)
    max_len -= strlen(readOnlyGlobals.hep.pwd)+sizeof(authkey_chunk);

  data_len = min(max_len, data_len);
  if(readOnlyGlobals.hep.sock == -1) return(-1);

  if(bkt->ext
     && bkt->ext->plugin 
     && (strcmp(bkt->ext->plugin->pluginPtr->short_name, "rtp") == 0)) {
    struct rtp_plugin_info *info = (struct rtp_plugin_info*)bkt->ext->plugin->pluginData;
    
    if((info->sip_call_id_computed) && (info->sip_call_id[0] != '\0')) {
      correlation_id = info->sip_call_id;
    } else {
      char ip_buf[32];

      snprintf(buf, sizeof(buf), "%s:%u", 
	       _intoaV4(bkt->core.tuple.key.k.ipKey.dst.ipType.ipv4, ip_buf, sizeof(ip_buf)),
	       bkt->core.tuple.key.k.ipKey.dport);
      correlation_id = buf;
    }
  }

  if(correlation_id == NULL) correlation_id = "";
  correlation_id_len = strlen(correlation_id);

  if((hg = malloc(sizeof(struct hep_generic))) == NULL)
    return(-2);

  memset(hg, 0, sizeof(struct hep_generic));

  /* header set */
  memcpy(hg->header.id, "\x48\x45\x50\x33", 4);

  /* IP proto */
  hg->ip_family.chunk.vendor_id = htons(0x0000);
  hg->ip_family.chunk.type_id   = htons(0x0001);
  hg->ip_family.data = (bkt->core.tuple.key.k.ipKey.src.ipVersion == 4) ? AF_INET : AF_INET6;
  hg->ip_family.chunk.length = htons(sizeof(hg->ip_family));

  /* Proto ID */
  hg->ip_proto.chunk.vendor_id = htons(0x0000);
  hg->ip_proto.chunk.type_id   = htons(0x0002);
  hg->ip_proto.data = bkt->core.tuple.key.k.ipKey.proto;
  hg->ip_proto.chunk.length = htons(sizeof(hg->ip_proto));

  /* IPv4 */
  if(bkt->core.tuple.key.k.ipKey.src.ipVersion == 4) {
    /* SRC IP */
    src_ip4.chunk.vendor_id = htons(0x0000);
    src_ip4.chunk.type_id   = htons(0x0003);
    src_ip4.data.s_addr = htonl(bkt->core.tuple.key.k.ipKey.src.ipType.ipv4);
    src_ip4.chunk.length = htons(sizeof(src_ip4));

    /* DST IP */
    dst_ip4.chunk.vendor_id = htons(0x0000);
    dst_ip4.chunk.type_id   = htons(0x0004);
    dst_ip4.data.s_addr = htonl(bkt->core.tuple.key.k.ipKey.dst.ipType.ipv4);
    dst_ip4.chunk.length = htons(sizeof(dst_ip4));

    iplen = sizeof(dst_ip4) + sizeof(src_ip4);
  }
#ifdef USE_IPV6
  /* IPv6 */
  else if(rcinfo->ip_family == AF_INET6) {
    /* SRC IPv6 */
    src_ip6.chunk.vendor_id = htons(0x0000);
    src_ip6.chunk.type_id   = htons(0x0005);
    memcpy(&src_ip6.data, &bkt->core.tuple.key.k.ipKey.src.ipType.ipv6, sizeof(struct in6_addr));
    src_ip6.chunk.length = htonl(sizeof(src_ip6));

    /* DST IPv6 */
    dst_ip6.chunk.vendor_id = htons(0x0000);
    dst_ip6.chunk.type_id   = htons(0x0006);
    memcpy(&dst_ip6.data, &bkt->core.tuple.key.k.ipKey.dst.ipType.ipv6, sizeof(struct in6_addr));
    dst_ip6.chunk.length = htonl(sizeof(dst_ip6));

    iplen = sizeof(dst_ip6) + sizeof(src_ip6);
  }
#endif

  /* SRC PORT */
  hg->src_port.chunk.vendor_id = htons(0x0000);
  hg->src_port.chunk.type_id   = htons(0x0007);
  hg->src_port.data = htons(bkt->core.tuple.key.k.ipKey.sport);
  hg->src_port.chunk.length = htons(sizeof(hg->src_port));

  /* DST PORT */
  hg->dst_port.chunk.vendor_id = htons(0x0000);
  hg->dst_port.chunk.type_id   = htons(0x0008);
  hg->dst_port.data = htons(bkt->core.tuple.key.k.ipKey.dport);
  hg->dst_port.chunk.length = htons(sizeof(hg->dst_port));

  /* TIMESTAMP SEC */
  hg->time_sec.chunk.vendor_id = htons(0x0000);
  hg->time_sec.chunk.type_id   = htons(0x0009);
  hg->time_sec.data = htonl(when);
  hg->time_sec.chunk.length = htons(sizeof(hg->time_sec));

  /* TIMESTAMP USEC */
  hg->time_usec.chunk.vendor_id = htons(0x0000);
  hg->time_usec.chunk.type_id   = htons(0x000a);
  hg->time_usec.data = htonl(0); /* no usec */
  hg->time_usec.chunk.length = htons(sizeof(hg->time_usec));

  /* Protocol TYPE */
  hg->proto_t.chunk.vendor_id = htons(0x0000);
  hg->proto_t.chunk.type_id   = htons(0x000b);
  hg->proto_t.data = 32 /* JSON */;
  hg->proto_t.chunk.length = htons(sizeof(hg->proto_t));

  /* Capture ID */
  hg->capt_id.chunk.vendor_id = htons(0x0000);
  hg->capt_id.chunk.type_id   = htons(0x000c);
  hg->capt_id.data = htons(readOnlyGlobals.hep.capture_id);
  hg->capt_id.chunk.length = htons(sizeof(hg->capt_id));

  /* Correlation ID */
  correlation_chunk.vendor_id = htons(0x0000);
  correlation_chunk.type_id   = htons(0x0011);
  correlation_chunk.length = htons(sizeof(hep_chunk_t) + correlation_id_len); //should be total lenght

/* Payload */
  payload_chunk.vendor_id = htons(0x0000);
  payload_chunk.type_id = sendzip ? htons(0x0010) : htons(0x000f);
  payload_chunk.length = htons(sizeof(payload_chunk) + data_len);

  tlen = sizeof(struct hep_generic) + data_len + iplen + 3*sizeof(hep_chunk_t) + correlation_id_len;

  /* auth key */
  if(readOnlyGlobals.hep.pwd != NULL) {
    tlen += sizeof(hep_chunk_t);

    /* Auth key */
    authkey_chunk.vendor_id = htons(0x0000);
    authkey_chunk.type_id   = htons(0x000e);
    authkey_chunk.length    = htons(sizeof(authkey_chunk) + strlen(readOnlyGlobals.hep.pwd));
    tlen += strlen(readOnlyGlobals.hep.pwd);
  }

  /* total */
  hg->header.length = htons(tlen);

  memcpy((void*) buffer, hg, sizeof(struct hep_generic));
  buflen = sizeof(struct hep_generic);

  free(hg);

  /* IPv4 */
  if(bkt->core.tuple.key.k.ipKey.src.ipVersion == 4) {
    /* SRC IP */
    memcpy((void*) buffer+buflen, &src_ip4, sizeof(struct hep_chunk_ip4));
    buflen += sizeof(struct hep_chunk_ip4);

    memcpy((void*) buffer+buflen, &dst_ip4, sizeof(struct hep_chunk_ip4));
    buflen += sizeof(struct hep_chunk_ip4);
  }
#ifdef USE_IPV6
  /* IPv6 */
  else if(src.ipVersion == 6) {
    /* SRC IPv6 */
    memcpy((void*) buffer+buflen, &src_ip4, sizeof(struct hep_chunk_ip6));
    buflen += sizeof(struct hep_chunk_ip6);

    memcpy((void*) buffer+buflen, &dst_ip6, sizeof(struct hep_chunk_ip6));
    buflen += sizeof(struct hep_chunk_ip6);
  }
#endif

  /* AUTH KEY CHUNK */
  if(readOnlyGlobals.hep.pwd != NULL) {
    memcpy((void*) buffer+buflen, &authkey_chunk,  sizeof(struct hep_chunk));
    buflen += sizeof(struct hep_chunk);

    /* Now copying payload self */
    memcpy((void*) buffer+buflen, readOnlyGlobals.hep.pwd, strlen(readOnlyGlobals.hep.pwd));
    buflen+=strlen(readOnlyGlobals.hep.pwd);
  }

  /* CORRELATION CHUNK */
  memcpy((void*) buffer+buflen, &correlation_chunk,  sizeof(struct hep_chunk));
  buflen +=  sizeof(struct hep_chunk);

  /* Now copying correlation_id self */
  memcpy((void*) buffer+buflen, correlation_id, correlation_id_len);
  buflen += correlation_id_len;

  /* PAYLOAD CHUNK */
  memcpy((void*) buffer+buflen, &payload_chunk,  sizeof(struct hep_chunk));
  buflen +=  sizeof(struct hep_chunk);

  /* Now copying payload self */
  memcpy((void*) buffer+buflen, data, data_len);
  buflen += data_len;

  /* send this packet out of our socket */
  if(sendto(readOnlyGlobals.hep.sock, buffer, buflen, 0 /* flags */,
	    (struct sockaddr *)&readOnlyGlobals.hep.server, sizeof(readOnlyGlobals.hep.server)) == -1) {
    traceEvent(TRACE_ERROR, "[HEP] [socket: %d][len: %u]", readOnlyGlobals.hep.sock, buflen);
    return(-3);
  } else
    return(buflen);
}

/* ***************************************************** */

int send_sip_hepv3(char *correlation_id, 
		   FlowHashBucket *bkt,
		   time_t when,
		   unsigned char *data, unsigned int data_len) {
  struct hep_generic *hg=NULL;
  char buffer[1500];
  unsigned int buflen=0, iplen=0, tlen=0;
  hep_chunk_ip4_t src_ip4, dst_ip4;
#ifdef USE_IPV6
  hep_chunk_ip6_t src_ip6, dst_ip6;
#endif
  hep_chunk_t payload_chunk;
  hep_chunk_t correlation_chunk;
  hep_chunk_t authkey_chunk;
  u_int correlation_id_len, max_len;
  unsigned int sendzip = 0;

  max_len = sizeof(buffer)-148;

  if(readOnlyGlobals.hep.pwd != NULL)
    max_len -= strlen(readOnlyGlobals.hep.pwd)+sizeof(authkey_chunk);

  data_len = min(max_len, data_len);
  if(readOnlyGlobals.hep.sock == -1) return(-1);

  if(correlation_id == NULL) correlation_id = "";
  correlation_id_len = strlen(correlation_id);

  hg = malloc(sizeof(struct hep_generic));
  memset(hg, 0, sizeof(struct hep_generic));

  /* header set */
  memcpy(hg->header.id, "\x48\x45\x50\x33", 4);

  /* IP proto */
  hg->ip_family.chunk.vendor_id = htons(0x0000);
  hg->ip_family.chunk.type_id   = htons(0x0001);
  hg->ip_family.data = (bkt->core.tuple.key.k.ipKey.src.ipVersion == 4) ? AF_INET : AF_INET6;
  hg->ip_family.chunk.length = htons(sizeof(hg->ip_family));

  /* Proto ID */
  hg->ip_proto.chunk.vendor_id = htons(0x0000);
  hg->ip_proto.chunk.type_id   = htons(0x0002);
  hg->ip_proto.data = bkt->core.tuple.key.k.ipKey.proto;
  hg->ip_proto.chunk.length = htons(sizeof(hg->ip_proto));

  /* IPv4 */
  if(bkt->core.tuple.key.k.ipKey.src.ipVersion == 4) {
    /* SRC IP */
    src_ip4.chunk.vendor_id = htons(0x0000);
    src_ip4.chunk.type_id   = htons(0x0003);
    src_ip4.data.s_addr = htonl(bkt->core.tuple.key.k.ipKey.src.ipType.ipv4);
    src_ip4.chunk.length = htons(sizeof(src_ip4));

    /* DST IP */
    dst_ip4.chunk.vendor_id = htons(0x0000);
    dst_ip4.chunk.type_id   = htons(0x0004);
    dst_ip4.data.s_addr = htonl(bkt->core.tuple.key.k.ipKey.dst.ipType.ipv4);
    dst_ip4.chunk.length = htons(sizeof(dst_ip4));

    iplen = sizeof(dst_ip4) + sizeof(src_ip4);
  }
#ifdef USE_IPV6
  /* IPv6 */
  else if(rcinfo->ip_family == AF_INET6) {
    /* SRC IPv6 */
    src_ip6.chunk.vendor_id = htons(0x0000);
    src_ip6.chunk.type_id   = htons(0x0005);
    memcpy(&src_ip6.data, &bkt->core.tuple.key.k.ipKey.src.ipType.ipv6, sizeof(struct in6_addr));
    src_ip6.chunk.length = htonl(sizeof(src_ip6));

    /* DST IPv6 */
    dst_ip6.chunk.vendor_id = htons(0x0000);
    dst_ip6.chunk.type_id   = htons(0x0006);
    memcpy(&dst_ip6.data, &bkt->core.tuple.key.k.ipKey.dst.ipType.ipv6, sizeof(struct in6_addr));
    dst_ip6.chunk.length = htonl(sizeof(dst_ip6));

    iplen = sizeof(dst_ip6) + sizeof(src_ip6);
  }
#endif

  /* SRC PORT */
  hg->src_port.chunk.vendor_id = htons(0x0000);
  hg->src_port.chunk.type_id   = htons(0x0007);
  hg->src_port.data = htons(bkt->core.tuple.key.k.ipKey.sport);
  hg->src_port.chunk.length = htons(sizeof(hg->src_port));

  /* DST PORT */
  hg->dst_port.chunk.vendor_id = htons(0x0000);
  hg->dst_port.chunk.type_id   = htons(0x0008);
  hg->dst_port.data = htons(bkt->core.tuple.key.k.ipKey.dport);
  hg->dst_port.chunk.length = htons(sizeof(hg->dst_port));

  /* TIMESTAMP SEC */
  hg->time_sec.chunk.vendor_id = htons(0x0000);
  hg->time_sec.chunk.type_id   = htons(0x0009);
  hg->time_sec.data = htonl(when);
  hg->time_sec.chunk.length = htons(sizeof(hg->time_sec));

  /* TIMESTAMP USEC */
  hg->time_usec.chunk.vendor_id = htons(0x0000);
  hg->time_usec.chunk.type_id   = htons(0x000a);
  hg->time_usec.data = htonl(when);
  hg->time_usec.chunk.length = htons(sizeof(hg->time_usec));

  /* Protocol TYPE */
  hg->proto_t.chunk.vendor_id = htons(0x0000);
  hg->proto_t.chunk.type_id   = htons(0x000b);
  hg->proto_t.data = 1 /* SIP */;
  hg->proto_t.chunk.length = htons(sizeof(hg->proto_t));

  /* Capture ID */
  hg->capt_id.chunk.vendor_id = htons(0x0000);
  hg->capt_id.chunk.type_id   = htons(0x000c);
  hg->capt_id.data = htons(readOnlyGlobals.hep.capture_id);
  hg->capt_id.chunk.length = htons(sizeof(hg->capt_id));

  /* Correlation ID */
  correlation_chunk.vendor_id = htons(0x0000);
  correlation_chunk.type_id   = htons(0x0011);
  correlation_chunk.length = htons(sizeof(hep_chunk_t) + correlation_id_len); //should be total lenght

/* Payload */
  payload_chunk.vendor_id = htons(0x0000);
  payload_chunk.type_id = sendzip ? htons(0x0010) : htons(0x000f);
  payload_chunk.length = htons(sizeof(payload_chunk) + data_len);

  tlen = sizeof(struct hep_generic) + data_len + iplen + 3*sizeof(hep_chunk_t) + correlation_id_len;

  /* auth key */
  if(readOnlyGlobals.hep.pwd != NULL) {
    tlen += sizeof(hep_chunk_t);
    /* Auth key */
    authkey_chunk.vendor_id = htons(0x0000);
    authkey_chunk.type_id   = htons(0x000e);
    authkey_chunk.length    = htons(sizeof(authkey_chunk) + strlen(readOnlyGlobals.hep.pwd));
    tlen += strlen(readOnlyGlobals.hep.pwd);
  }

  /* total */
  hg->header.length = htons(tlen);

  memcpy((void*) buffer, hg, sizeof(struct hep_generic));
  buflen = sizeof(struct hep_generic);

  free(hg);

  /* IPv4 */
  if(bkt->core.tuple.key.k.ipKey.src.ipVersion == 4) {
    /* SRC IP */
    memcpy((void*) buffer+buflen, &src_ip4, sizeof(struct hep_chunk_ip4));
    buflen += sizeof(struct hep_chunk_ip4);

    memcpy((void*) buffer+buflen, &dst_ip4, sizeof(struct hep_chunk_ip4));
    buflen += sizeof(struct hep_chunk_ip4);
  }
#ifdef USE_IPV6
  /* IPv6 */
  else if(src->ipVersion == 6) {
    /* SRC IPv6 */
    memcpy((void*) buffer+buflen, &src_ip4, sizeof(struct hep_chunk_ip6));
    buflen += sizeof(struct hep_chunk_ip6);

    memcpy((void*) buffer+buflen, &dst_ip6, sizeof(struct hep_chunk_ip6));
    buflen += sizeof(struct hep_chunk_ip6);
  }
#endif

  /* AUTH KEY CHUNK */
  if(readOnlyGlobals.hep.pwd != NULL) {
    memcpy((void*) buffer+buflen, &authkey_chunk,  sizeof(struct hep_chunk));
    buflen += sizeof(struct hep_chunk);

    /* Now copying payload self */
    memcpy((void*) buffer+buflen, readOnlyGlobals.hep.pwd, strlen(readOnlyGlobals.hep.pwd));
    buflen+=strlen(readOnlyGlobals.hep.pwd);
  }

  /* CORRELATION CHUNK */
  memcpy((void*) buffer+buflen, &correlation_chunk,  sizeof(struct hep_chunk));
  buflen +=  sizeof(struct hep_chunk);

  /* Now copying correlation_id self */
  memcpy((void*) buffer+buflen, correlation_id, correlation_id_len);
  buflen += correlation_id_len;

  /* PAYLOAD CHUNK */
  memcpy((void*) buffer+buflen, &payload_chunk,  sizeof(struct hep_chunk));
  buflen +=  sizeof(struct hep_chunk);

  /* Now copying payload self */
  memcpy((void*) buffer+buflen, data, data_len);
  buflen += data_len;

  /* send this packet out of our socket */
  if(sendto(readOnlyGlobals.hep.sock, buffer, buflen, 0 /* flags */,
	    (struct sockaddr *)&readOnlyGlobals.hep.server, sizeof(readOnlyGlobals.hep.server)) == -1) {
    traceEvent(TRACE_ERROR, "[HEP] [socket: %d][len: %u]", readOnlyGlobals.hep.sock, buflen);
    return(-3);
  } else
    return(buflen);
}

/* ***************************************************** */
/* SIP Call-ID + MOS + R-FACTOR for RTP Reports in HEP3  */
/* ***************************************************** */

int send_mos_hepv3(char *correlation_id, 
		   FlowHashBucket *bkt,
		   time_t when,
		   unsigned int mos, unsigned int rfactor) {
  struct hep_mos *hg = NULL;
  char buffer[1500];
  unsigned int buflen=0, iplen=0, tlen=0;
  hep_chunk_t correlation_chunk;
  hep_chunk_t authkey_chunk;
  u_int correlation_id_len;

  if(readOnlyGlobals.hep.sock == -1) return(-1);

  /* if(correlation_id == NULL) correlation_id = ""; */
  if(correlation_id == NULL) return(-1);
  correlation_id_len = strlen(correlation_id);

  hg = malloc(sizeof(struct hep_mos));
  memset(hg, 0, sizeof(struct hep_mos));

  /* header set */
  memcpy(hg->header.id, "\x48\x45\x50\x33", 4);

  /* TIMESTAMP SEC */
  hg->time_sec.chunk.vendor_id = htons(0x0000);
  hg->time_sec.chunk.type_id   = htons(0x0009);
  hg->time_sec.data = htonl(when);
  hg->time_sec.chunk.length = htons(sizeof(hg->time_sec));

  /* TIMESTAMP USEC */
  hg->time_usec.chunk.vendor_id = htons(0x0000);
  hg->time_usec.chunk.type_id   = htons(0x000a);
  hg->time_usec.data = htonl(when);
  hg->time_usec.chunk.length = htons(sizeof(hg->time_usec));

  /* Protocol TYPE */
  hg->proto_t.chunk.vendor_id = htons(0x0000);
  hg->proto_t.chunk.type_id   = htons(0x000b);
  hg->proto_t.data = 33 /* MOS REPORTS */;
  hg->proto_t.chunk.length = htons(sizeof(hg->proto_t));

  /* Capture ID */
  hg->capt_id.chunk.vendor_id = htons(0x0000);
  hg->capt_id.chunk.type_id   = htons(0x000c);
  hg->capt_id.data = htons(readOnlyGlobals.hep.capture_id);
  hg->capt_id.chunk.length = htons(sizeof(hg->capt_id));
  
    /* MOS */
  hg->mos.chunk.vendor_id = htons(0x0007);
  hg->mos.chunk.type_id   = htons(0x0020);
  hg->mos.data = htons(mos);
  hg->mos.chunk.length = htons(sizeof(u_int16_t));

  /* R-FACTOR */
  hg->rfactor.chunk.vendor_id = htons(0x0007);
  hg->rfactor.chunk.type_id   = htons(0x0021);
  hg->rfactor.data = htons(rfactor);
  hg->rfactor.chunk.length = htons(sizeof(u_int16_t));

  /* Correlation ID */
  correlation_chunk.vendor_id = htons(0x0000);
  correlation_chunk.type_id   = htons(0x0011);
  correlation_chunk.length = htons(sizeof(hep_chunk_t) + correlation_id_len); //should be total lenght

  tlen = sizeof(struct hep_mos) + iplen + sizeof(hep_chunk_t) + correlation_id_len;

  /* auth key */
  if(readOnlyGlobals.hep.pwd != NULL) {
    tlen += sizeof(hep_chunk_t);
    /* Auth key */
    authkey_chunk.vendor_id = htons(0x0000);
    authkey_chunk.type_id   = htons(0x000e);
    authkey_chunk.length    = htons(sizeof(authkey_chunk) + strlen(readOnlyGlobals.hep.pwd));
    tlen += strlen(readOnlyGlobals.hep.pwd);
  }

  /* total */
  hg->header.length = htons(tlen);

  memcpy((void*) buffer, hg, sizeof(struct hep_mos));
  buflen = sizeof(struct hep_mos);

  free(hg);

  /* AUTH KEY CHUNK */
  if(readOnlyGlobals.hep.pwd != NULL) {
    memcpy((void*) buffer+buflen, &authkey_chunk,  sizeof(struct hep_chunk));
    buflen += sizeof(struct hep_chunk);

    /* Now copying payload self */
    memcpy((void*) buffer+buflen, readOnlyGlobals.hep.pwd, strlen(readOnlyGlobals.hep.pwd));
    buflen+=strlen(readOnlyGlobals.hep.pwd);
  }

  /* CORRELATION CHUNK */
  memcpy((void*) buffer+buflen, &correlation_chunk,  sizeof(struct hep_chunk));
  buflen +=  sizeof(struct hep_chunk);

  /* Now copying correlation_id self */
  memcpy((void*) buffer+buflen, correlation_id, correlation_id_len);
  buflen += correlation_id_len;

  /* send this packet out of our socket */
  if(sendto(readOnlyGlobals.hep.sock, buffer, buflen, 0 /* flags */,
	    (struct sockaddr *)&readOnlyGlobals.hep.server, sizeof(readOnlyGlobals.hep.server)) == -1) {
    traceEvent(TRACE_ERROR, "[HEP-MOS] [socket: %d][len: %u]", readOnlyGlobals.hep.sock, buflen);
    return(-3);
  } else
    return(buflen);
}

