/* HEPv3 types */

struct hep_chunk {
  u_int16_t vendor_id;
  u_int16_t type_id;
  u_int16_t length;
} __attribute__((packed));

typedef struct hep_chunk hep_chunk_t;

struct hep_chunk_uint8 {
  hep_chunk_t chunk;
  u_int8_t data;
} __attribute__((packed));

typedef struct hep_chunk_uint8 hep_chunk_uint8_t;

struct hep_chunk_uint16 {
  hep_chunk_t chunk;
  u_int16_t data;
} __attribute__((packed));

typedef struct hep_chunk_uint16 hep_chunk_uint16_t;

struct hep_chunk_uint32 {
  hep_chunk_t chunk;
  u_int32_t data;
} __attribute__((packed));

typedef struct hep_chunk_uint32 hep_chunk_uint32_t;

struct hep_chunk_str {
  hep_chunk_t chunk;
  char *data;
} __attribute__((packed));

typedef struct hep_chunk_str hep_chunk_str_t;

struct hep_chunk_ip4 {
  hep_chunk_t chunk;
  struct in_addr data;
} __attribute__((packed));

typedef struct hep_chunk_ip4 hep_chunk_ip4_t;

struct hep_chunk_ip6 {
  hep_chunk_t chunk;
  struct in6_addr data;
} __attribute__((packed));

typedef struct hep_chunk_ip6 hep_chunk_ip6_t;

struct hep_ctrl {
  char id[4];
  u_int16_t length;
} __attribute__((packed));

typedef struct hep_ctrl hep_ctrl_t;

struct hep_chunk_payload {
  hep_chunk_t chunk;
  char *data;
} __attribute__((packed));

typedef struct hep_chunk_payload hep_chunk_payload_t;

/* Structure of HEP */

struct hep_generic {
  hep_ctrl_t         header;
  hep_chunk_uint8_t  ip_family;
  hep_chunk_uint8_t  ip_proto;
  hep_chunk_uint16_t src_port;
  hep_chunk_uint16_t dst_port;
  hep_chunk_uint32_t time_sec;
  hep_chunk_uint32_t time_usec;
  hep_chunk_uint8_t  proto_t;
  hep_chunk_uint32_t capt_id;
} __attribute__((packed));

typedef struct hep_generic hep_generic_t;

struct hep_mos {
  hep_ctrl_t         header;
  hep_chunk_uint32_t time_sec;
  hep_chunk_uint32_t time_usec;
  hep_chunk_uint8_t  proto_t;
  hep_chunk_uint16_t mos;
  hep_chunk_uint16_t rfactor;    
  hep_chunk_uint32_t capt_id;
} __attribute__((packed));

typedef struct hep_mos hep_mos_t;


/* ****************************************** */

extern int send_json_hepv3(FlowHashBucket *bkt, time_t when,
			   char *data, unsigned int data_len);

extern int send_sip_hepv3(char *correlation_id,
			  FlowHashBucket *bkt,
			  time_t when, 
			  unsigned char *data, unsigned int data_len);

extern int send_mos_hepv3(char *correlation_id, 
			  FlowHashBucket *bkt,
			  time_t when,
			  unsigned int mos, unsigned int rfactor);
