
/*
  http://myaccount.flukenetworks.com/fnet/en-us/supportAndDownloads/KB/IT+Networking/protocol+expert/What_is_an_R-Factor

  Desirability Scale              R-factor          MOS Value
  Desirable                       94 – 80           4.4 – 4.0
  Acceptable                      80 – 70           4.0 – 3.6
  Reach Connection                70 – 50           3.6 – 2.6
  Not Recommended                 50 – 0            2.6 – 0
*/

double mean_opinion_score(FlowDirection direction, double rtt, double jitter,
			  u_int32_t num_packets_lost, double *r_factor) {
  /*
    Take the rtt latency, add jitter, but double the impact to latency
    then add 10 for protocol latancies
  */
  double effective_latency = rtt + (jitter * 2) + 10;
  double mos_val;

  /*
    Implement a basic curve - deduct 4 for the r_factor at 160ms of latency
    (round trip). Anything over that gets a much more agressive deduction
  */
  if(effective_latency < 160) {
    *r_factor = 93.2 - (effective_latency / 40);
  } else {
    *r_factor = 93.2 - (effective_latency - 120) / 10;
  }

  /*
    Now, let's deduct 2.5 r_factor per percentage of num_packets_lost
  */
  *r_factor = *r_factor - (num_packets_lost * 2.5);
  if(*r_factor > 100) *r_factor = 100;
  else if(*r_factor < 0) *r_factor = 0;

  /* Convert the r_factor into an MOS value. (this is a known formula) */
  mos_val = 1 + (0.035) * (*r_factor) +
    (0.000007) * (*r_factor) * ((*r_factor) - 60)
    * (100 - (*r_factor));

  if(mos_val > 5) mos_val = 5;

  traceEvent(TRACE_INFO, "[%s][RTT: %.2f][Jitter: %.2f]"
	     "[# Packet Lost: %u][R-Factor: %.2f][MOS: %.2f]",
	     (direction == src2dst_direction) ? "src2dst" : "dst2src",
	     rtt, jitter, num_packets_lost, *r_factor, mos_val);

  return(mos_val);
}

