/*
 *  Copyright (C) 2007 Luca Deri <deri@ntop.org>
 *
 *  		       http://www.ntop.org/
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef _PF_RING_H_
#define _PF_RING_H_

#ifdef HAVE_PF_RING

#ifndef __USE_XOPEN2K
#define __USE_XOPEN2K
#endif

#define HAVE_PCAP /* Needed for pfring.h */
#undef __USE_XOPEN2K
#include "pfring.h"

/* PF_RING hook */
extern void* fetchPfRingPackets(void* notUsed);

/* ******************************************* */
/* ******************************************* */

/* Process Plugin */

#include "uthash.h"

#undef uthash_fatal
#define uthash_fatal(a) { }

struct uid_name {
  u_int32_t uid; /* Key: it must be 32 bit although 16 bits are enough */
  u_int16_t gid;
  char *user_name;
  UT_hash_handle hh; /* makes this structure hashable */
};

/* ******************************************* */

struct port_inode {
  u_int16_t local_port, remote_port;
  u_int8_t is_tcp, ip_version;
  u_int64_t inode;
  UT_hash_handle hh; /* makes this structure hashable */
};

/* ******************************************* */

struct fd_type {
  u_int32_t fd_pid; /* fd:16, pid=16 */
  u_int socket_type;
  UT_hash_handle hh; /* makes this structure hashable */
};

/* ******************************************* */

struct tid_pid_exe {
  u_int32_t tid; /* Key: it must be 32 bit although 16 bits are enough */
  u_int16_t pid, father_pid, uid, gid;
  char *exe;
  time_t expire_time; /* Delayed delete */
  UT_hash_handle hh; /* makes this structure hashable */
};

/* ******************************************* */

struct port_exe {
  u_int32_t port; /* Key: it must be 32 bit although 16 bits are enough */
  u_int16_t pid, father_pid;
  char *exe;
  time_t expire_time; /* Delayed delete */
  UT_hash_handle hh; /* makes this structure hashable */
};

/* ******************************************* */

struct fd_pid {
  u_int32_t fd_pid; /* fd:16, pid=16 */
  u_int8_t is_tcp, ip_version;
  u_int16_t local_port, remote_port, tid;
  char *process_name;
  time_t when;
  UT_hash_handle hh; /* makes this structure hashable */
};

/* ******************************************* */
/* ******************************************* */

#endif /* HAVE_PF_RING */

#endif /* _PF_RING_H_ */
